import os.path

import jinja2.visitor
import jinja2.compiler
import jinja2.environment

import pwt.jinja2js.jscompiler
import pwt.jinja2js.environment
import pwt.jinja2js.nodes

import files

class Source(jinja2.visitor.NodeVisitor):
    """
    Source object for Jinja2 templates that we can compile to Java Script

    We require a Jinja2 environment in order to start to compile the templates.

    
    """

    def __init__(self, tree, path):
        self._tree = tree
        self.env = tree.config["jinja2.environment"]

        self.source = files.source.GetFileContents(path).decode("utf-8")
        self.node = self.env._parse(
            self.source,
            os.path.basename(path),
            path)

        self._path = path

        self.provides = set([])
        # Manually added
        self.requires = set(["goog.string", "goog.string.StringBuffer"])
        self.visit(self.node)

    def GetSource(self):
        return pwt.jinja2js.jscompiler.generateClosure(
            self.node,
            self.env,
            os.path.basename(self._path),
            self._path)

    def GetPath(self):
        # I want files.py to call GetSource when compiling Java Script templates
        return None

    def visit_NamespaceNode(self, node):
        self.provides.add(node.namespace.encode("utf-8"))

    def visit_Import(self, node):
        name = node.template.value
        source, filename, uptodate = self.env.loader.get_source(self.env, name)
        fromnode = self.env._parse(source, name, filename)

        # Need to find the namespace
        namespace = list(fromnode.find_all(pwt.jinja2js.nodes.NamespaceNode))
        if len(namespace) != 1:
            raise jinja2.compiler.TemplateAssertionError(
                "You must supply one namespace for your template",
                0,
                name,
                filename)
        namespace = namespace[0].namespace

        self.requires.add(namespace.encode("utf-8"))



import json
import os
import urlparse
import datetime

import webob.dec
import webob.datetime_utils
import paste.urlmap

RAW = "RAW"
SIMPLE = "SIMPLE"
ADVANCED = "ADVANCED"

import files

JS_CONTENT_TYPE = "application/javascript"

class Input(object):

    def __init__(self, tree, **kwargs):
        self.tree = tree

    @webob.dec.wsgify
    def __call__(self, request):
        try:
            src = self.tree.getSource(request.path_info)
        except KeyError:
            return webob.Response(
                status = 404,
                content_type = None)
        else:
            # Use the file stats to see if we need to send the contents of the
            # file back to the client.
            last_modified = datetime.datetime.fromtimestamp(
                int(src.mtime), webob.datetime_utils.UTC)
            if request.if_modified_since and \
                   last_modified <= request.if_modified_since:
                return webob.Response(
                    status = 304,
                    content_type = JS_CONTENT_TYPE,
                    last_modified = src.mtime)

            return webob.Response(
                body = src.GetSource(), # May call external programs
                status = 200,
                content_type = JS_CONTENT_TYPE,
                last_modified = src.mtime)


class Raw(object):

    def __init__(self, tree, **kwargs):
        self.tree = tree

    @webob.dec.wsgify
    def __call__(self, request):
        deps = self.tree.getDeps(request.GET.getall("input"))
        path = "/compile"

        base_url = urlparse.urljoin(request.url, "input/")
        # remove starting '/'
        jsfiles = [
            urlparse.urljoin(base_url, js_source.path_info[1:])
            for js_source in deps
            ]

        # Simple small Java Script snippet that checks to see if it is included
        # on this page and then inserts all the dependencies on the inputs.
        output = """(function() {
    CLOSURE_NO_DEPS = true;

    var files = %s;
    var path = '%s';

    var scriptEl;
    var doc = document;
    var scripts = doc.getElementsByTagName('script');
    for (var i = scripts.length - 1; i >= 0; --i) {
        var candidateScriptEl = scripts[i];
        var src = candidateScriptEl.src;
        if (src.indexOf(path) >= 0) {
            scriptEl = candidateScriptEl;
            break;
        }
    }

    if (!scriptEl) {
        return;
    }

    for (var i = 0; i < files.length; i++) {
        doc.write('<script type="text/javascript" src="' + files[i] + '"><\/script>');
    }
})();
""" %(json.dumps(jsfiles), path)

        return webob.Response(
            body = output, content_type = "application/javascript")


class Compile(object):

    def __init__(self, tree, **kwargs):
        self.tree = tree

    @webob.dec.wsgify
    def __call__(self, request):
        output = self.tree.getCompiledSource(request.GET.getall("input"))

        return webob.Response(
            body = output, content_type = "application/javascript")


class Deps(object):

    def __init__(self, tree, **kwargs):
        self.tree = tree

    @webob.dec.wsgify
    def __call__(self, request):
        # XXX - generate an unique ID on start up and use this as a Etag
        # to perform caching on this file.
        base_path = urlparse.urljoin(request.path, "input/")
        output = files.MakeDepsFile(self.tree, base_path)

        return webob.Response(
            body = output, content_type = "application/javascript")


class Combined(object):
    """
    This is the main WSGI entry point
    """

    def __init__(self, **local_conf):
        self.app = paste.urlmap.URLMap()
        self.app["/compile"] = Raw(**local_conf)
        self.app["/input"] = Input(**local_conf)
        self.app["/deps"] = Deps(**local_conf)

    def __call__(self, environ, start_response):
        return self.app(environ, start_response)


def get_input_arguments(local_conf):
    # local_conf is a dictionary of unparsed configuration from an ini file.
    # We need to split some values up into lists where appropriate.
    inputs = local_conf.get("inputs", "")
    if inputs:
        inputs = inputs.split()
    else:
        inputs = ()
    local_conf["inputs"] = inputs

    # need to configure Jinja2 environment if appropriate
    try:
        local_conf["jinja2.environment"] = files.parse_environment(local_conf)
    except (KeyError, ValueError), err:
        pass

    local_conf["paths"] = local_conf["paths"].split()

    local_conf["tree"] = files.Tree(**local_conf)

    return local_conf

def paste_combined_closure(global_conf, **local_conf):
    conf = get_input_arguments(local_conf)

    return Combined(tree = conf["tree"])



import logging
import os.path
import urlparse
import re
import shutil
import string
import subprocess
import sys
import tempfile

# We need to reload the server whenever any of our files change. We need
# to support multiple monitors here.
_reloaders = []
try:
    import paste.reloader
except ImportError:
    pass
else:
    _reloaders.append(paste.reloader.watch_file)

try:
    import pyramid.scripts.pserve
except ImportError:
    pass
else:
    _reloaders.append(pyramid.scripts.pserve.watch_file)


# Import the depswrite and source from closure-library checkout
old_path = sys.path
sys.path.append(os.path.join(os.path.dirname(__file__), "build"))
import depswriter
import depstree
import source
import treescan
import closurebuilder
import jscompiler
# reset the path
sys.path = sys.path[:-1]


_FILE_REGEX = re.compile(r"^.+\.(js|soy|jinja2)$")

DEFAULT_SOY_JAR = os.path.join(
    os.path.dirname(__file__), "jars", "SoyToJsSrcCompiler.jar")

DEFAULT_COMPILER_JAR = os.path.join(
    os.path.dirname(__file__), "jars", "compiler.jar")


class PathSource(source.Source):
    """
    Source object of ordinary Java Script file
    """

    def __init__(self, tree, path):
        super(PathSource, self).__init__(source.GetFileContents(path))

        self._path = path

    def GetPath(self):
        return self._path


_SOY_NAMESPACE = re.compile(r"{namespace\s+([a-zA-Z0-9_\.]+)\s*}")

class SoySource(source.Source):
    """
    Source object of Closure Template file. GetSource converts template
    to Java Script
    """

    def __init__(self, tree, path):
        self.provides = set()
        self.requires = set()

        self._tree = tree

        self._path = None
        self._src = path
        self._ScanSource()

        # We need to collect all templates and generate them
        # all at once so that we only call a java subprocess once
        soyes = getattr(tree, "_soyes", None)
        if soyes is None:
            tree._soyes = set()
            tree._soyes_built = False
        tree._soyes.add(self)

    def GetPath(self):
        return self._path

    def GetSource(self):
        self._update()
        return self._source

    def _update(self):
        if not self._tree._soyes_built and self._tree._soyes:
            self._build_soyes()

    def _build_soyes(self):
        soyes = self._tree._soyes

        # XXX - we could have multiple files with the same filename but
        # located in a different patch that will override each other here.
        # XXX - Also we need to be able to handle multiple languages
        tmpdir = tempfile.mkdtemp()
        outputPathFormat = "%s/{INPUT_FILE_NAME_NO_EXT}.js" % tmpdir
        args = [
            "java", "-jar", self._tree.config["soy_jar"],
            "--shouldProvideRequireSoyNamespaces",
            "--outputPathFormat", outputPathFormat,
            ]
        for soy in soyes:
            args.append(soy._src)

        proc = subprocess.Popen(args, stdout = subprocess.PIPE)
        stdoutdata, unused_stderrdata = proc.communicate()

        if proc.returncode != 0:
            raise ValueError("Failed to generate templates")

        for soy in soyes:
            # Patch up the source
            _path = get_output_filename(outputPathFormat, soy._src)
            soy._source = source.GetFileContents(_path)

        # Cleanup
        shutil.rmtree(tmpdir)

        self._tree._soyes_built = True

    def _ScanSource(self):
        src = source.GetFileContents(self._src)

        self.requires.add("soy")
        self.requires.add("soy.StringBuilder")

        src_lines = src.splitlines()
        for line in src_lines:
            match = _SOY_NAMESPACE.match(line)
            if match:
                self.provides.add(match.group(1))

            # XXX - need to implement a RE for any {call}...{/call} blocks
            # that would define a goog.require for us


def get_output_filename(output_format, filename):
    filename_template = string.Template(output_format)

    data = {
        "{INPUT_FILE_NAME}": os.path.basename(filename),
        "{INPUT_FILE_NAME_NO_EXT}": os.path.splitext(os.path.basename(filename))[0],
        "{INPUT_DIRECTORY}": os.path.dirname(filename),
        # "{INPUT_PREFIX}": 
        }

    for key, val in data.items():
        output_format = output_format.replace(key, val)

    return output_format


_default_config = {
    # SoyToJsSrcCompiler.jar
    "soy_jar": DEFAULT_SOY_JAR,
    # compiler.jar
    "compiler_jar": DEFAULT_COMPILER_JAR,
    "compiler_flags": [],
    # default inputs
    "inputs": [],
    }

_default_config["plugins_extensions"] = {
    ".js": PathSource,
    ".soy": SoySource,
    }
try:
    import jinja
    import pwt.jinja2js.environment
except ImportError: # pwt.jinja2js not installed, probable
    def parse_environment(settings):
        raise ValueError("pwt.jinja2js is not installed")
else:
    _default_config["plugins_extensions"][".jinja2"] = jinja.Source

    def parse_environment(settings):
        return pwt.jinja2js.environment.parse_environment(settings)


class Tree(object):

    def __init__(self, **config):
        self.roots = config["paths"]
        # Extend the default configuration but don't change it
        self.config = _default_config.copy()
        self.config.update(config)

        self.tree = self.base = self.path_info = None

    def update(self):
        if self.tree is not None:
            return

        sources = set()
        path_info = {}
        basefile = None

        for root in self.roots:
            root = os.path.abspath(root)

            for jsfile in treescan.ScanTree(root, _FILE_REGEX):
                ext = os.path.splitext(jsfile)[1]
                # throws KeyError if we don't understand the file extension
                src = _default_config["plugins_extensions"][ext](self, jsfile)

                if basefile is None and closurebuilder._IsClosureBaseFile(src):
                    basefile = src

                # Save the path_info so that we can map path_info to a source
                # object and back.
                path = jsfile[len(root):]
                path_info[path] = src
                src.path_info = path
                # Rememmber the last modified time so that we can before some
                # sort of caching of resources.
                src.mtime = os.stat(jsfile).st_mtime

                # watch file for changes. This reloads the whole server
                # compiling any templates that need to changed.
                # This does nothing if we are not running under the PasteScript
                # server that stops and reloads whenever any changes are found.
                [watch_file(jsfile) for watch_file in _reloaders]

                sources.add(src)

        if basefile is None:
            raise ValueError("No Closure base.js found")

        self.tree = depstree.DepsTree(sources)

        self.base = basefile

        self.path_info = path_info

    def getDeps(self, inputs = None):
        # Returns a list of Source objects.
        self.update()
        inputs = inputs or self.config["inputs"]

        input_namespaces = set()
        for input_path in inputs:
            src = None
            try:
                js_input = source.Source(source.GetFileContents(input_path))
            except IOError:
                # All path_info attributes start with a slash. So put in
                # the slash to make sure we find the file
                js_input = self.getSource(os.path.join("/", input_path))
            
            input_namespaces.update(js_input.provides)
        if not input_namespaces:
            raise ValueError("Input namespaces must be specified")

        deps = [self.base] + self.tree.GetDependencies(input_namespaces)

        return deps

    def getCompiledSource(self, inputs = None):
        deps = self.getDeps(inputs)

        # Collect all the temporary files and remove them at the end in a
        # finally statement.
        alltempfiles = []

        try:
            args = ["java", "-jar", self.config["compiler_jar"]]
            for src in deps:
                path = src.GetPath()
                if path is None:
                    # Soy Source files create tempfiles so we need to recreate
                    # them now so that we don't leave loads of tempfiles on the
                    # system
                    # tp - tempfile that gets cleared out when the pointer goes
                    # out of scope
                    fp = tempfile.NamedTemporaryFile(
                        suffix = ".js", delete = False)
                    fp.write(src.GetSource())
                    fp.close()
                    path = fp.name

                    alltempfiles.append(path)

                args += ["--js", path]

            args += self.config["compiler_flags"]

            logging.info(
                "Compiling with the following command: %s", " ".join(args))

            proc = subprocess.Popen(args, stdout = subprocess.PIPE)
            stdoutdata, unused_stderrdata = proc.communicate()

            if proc.returncode != 0:
                raise ValueError("Compilation failed")

            return stdoutdata
        finally:
            for tmp in alltempfiles:
                os.remove(tmp)

    def getSource(self, path_info):
        self.update()
        # Note: get the Java Script source
        return self.path_info[path_info]


def MakeDepsFile(tree, base_path = ""):
    """
    Convert the Tree object into a deps.js file. Returns the contents of
    such a file.

    Note that the base_url shouldn't contain the host. As then goog.require
    we break due to limitations in its implementation.
    """
    tree.update()

    # We remove the starting '/' on all the path_info
    source_map = dict([
        (urlparse.urljoin(base_path, src.path_info[1:]), src)
        for src in tree.tree._sources
        ])

    return depswriter.MakeDepsFile(source_map)



import os.path
import shutil
import tempfile
import unittest

import paste.urlmap
import webtest
import zc.buildout.buildout
import zc.buildout.testing

import wsgi
import files

BODIES = {
    "test1": """(function() {
    CLOSURE_NO_DEPS = true;

    var files = ["http://localhost/input/closure/goog/base.js", "http://localhost/input/closure/goog/string/stringbuffer.js", "http://localhost/input/closure/goog/i18n/bidi.js", "http://localhost/input/closure/goog/debug/error.js", "http://localhost/input/closure/goog/string/string.js", "http://localhost/input/closure/goog/asserts/asserts.js", "http://localhost/input/closure/goog/array/array.js", "http://localhost/input/closure/goog/dom/classes.js", "http://localhost/input/closure/goog/object/object.js", "http://localhost/input/closure/goog/dom/tagname.js", "http://localhost/input/closure/goog/useragent/useragent.js", "http://localhost/input/closure/goog/math/size.js", "http://localhost/input/closure/goog/dom/browserfeature.js", "http://localhost/input/closure/goog/math/math.js", "http://localhost/input/closure/goog/math/coordinate.js", "http://localhost/input/closure/goog/dom/dom.js", "http://localhost/input/closure/goog/structs/inversionmap.js", "http://localhost/input/closure/goog/i18n/graphemebreak.js", "http://localhost/input/closure/goog/format/format.js", "http://localhost/input/closure/goog/i18n/bidiformatter.js", "http://localhost/input/soyutils_usegoog.js", "http://localhost/input/test1.soy", "http://localhost/input/test1.js"];
    var path = '/compile';

    var scriptEl;
    var doc = document;
    var scripts = doc.getElementsByTagName('script');
    for (var i = scripts.length - 1; i >= 0; --i) {
        var candidateScriptEl = scripts[i];
        var src = candidateScriptEl.src;
        if (src.indexOf(path) >= 0) {
            scriptEl = candidateScriptEl;
            break;
        }
    }

    if (!scriptEl) {
        return;
    }

    for (var i = 0; i < files.length; i++) {
        doc.write('<script type="text/javascript" src="' + files[i] + '"><\/script>');
    }
})();
""",

    "test2": """(function() {
    CLOSURE_NO_DEPS = true;

    var files = ["http://localhost/input/closure/goog/base.js", "http://localhost/input/closure/goog/string/stringbuffer.js", "http://localhost/input/closure/goog/i18n/bidi.js", "http://localhost/input/closure/goog/debug/error.js", "http://localhost/input/closure/goog/string/string.js", "http://localhost/input/closure/goog/asserts/asserts.js", "http://localhost/input/closure/goog/array/array.js", "http://localhost/input/closure/goog/dom/classes.js", "http://localhost/input/closure/goog/object/object.js", "http://localhost/input/closure/goog/dom/tagname.js", "http://localhost/input/closure/goog/useragent/useragent.js", "http://localhost/input/closure/goog/math/size.js", "http://localhost/input/closure/goog/dom/browserfeature.js", "http://localhost/input/closure/goog/math/math.js", "http://localhost/input/closure/goog/math/coordinate.js", "http://localhost/input/closure/goog/dom/dom.js", "http://localhost/input/closure/goog/structs/inversionmap.js", "http://localhost/input/closure/goog/i18n/graphemebreak.js", "http://localhost/input/closure/goog/format/format.js", "http://localhost/input/closure/goog/i18n/bidiformatter.js", "http://localhost/input/soyutils_usegoog.js", "http://localhost/input/test1.soy", "http://localhost/input/test2.js", "http://localhost/input/test1.js"];
    var path = '/compile';

    var scriptEl;
    var doc = document;
    var scripts = doc.getElementsByTagName('script');
    for (var i = scripts.length - 1; i >= 0; --i) {
        var candidateScriptEl = scripts[i];
        var src = candidateScriptEl.src;
        if (src.indexOf(path) >= 0) {
            scriptEl = candidateScriptEl;
            break;
        }
    }

    if (!scriptEl) {
        return;
    }

    for (var i = 0; i < files.length; i++) {
        doc.write('<script type="text/javascript" src="' + files[i] + '"><\/script>');
    }
})();
""",

    "test1_js": """(function() {
    CLOSURE_NO_DEPS = true;

    var files = ["http://localhost/js/input/closure/goog/base.js", "http://localhost/js/input/closure/goog/string/stringbuffer.js", "http://localhost/js/input/closure/goog/i18n/bidi.js", "http://localhost/js/input/closure/goog/debug/error.js", "http://localhost/js/input/closure/goog/string/string.js", "http://localhost/js/input/closure/goog/asserts/asserts.js", "http://localhost/js/input/closure/goog/array/array.js", "http://localhost/js/input/closure/goog/dom/classes.js", "http://localhost/js/input/closure/goog/object/object.js", "http://localhost/js/input/closure/goog/dom/tagname.js", "http://localhost/js/input/closure/goog/useragent/useragent.js", "http://localhost/js/input/closure/goog/math/size.js", "http://localhost/js/input/closure/goog/dom/browserfeature.js", "http://localhost/js/input/closure/goog/math/math.js", "http://localhost/js/input/closure/goog/math/coordinate.js", "http://localhost/js/input/closure/goog/dom/dom.js", "http://localhost/js/input/closure/goog/structs/inversionmap.js", "http://localhost/js/input/closure/goog/i18n/graphemebreak.js", "http://localhost/js/input/closure/goog/format/format.js", "http://localhost/js/input/closure/goog/i18n/bidiformatter.js", "http://localhost/js/input/soyutils_usegoog.js", "http://localhost/js/input/test1.soy", "http://localhost/js/input/test1.js"];
    var path = '/compile';

    var scriptEl;
    var doc = document;
    var scripts = doc.getElementsByTagName('script');
    for (var i = scripts.length - 1; i >= 0; --i) {
        var candidateScriptEl = scripts[i];
        var src = candidateScriptEl.src;
        if (src.indexOf(path) >= 0) {
            scriptEl = candidateScriptEl;
            break;
        }
    }

    if (!scriptEl) {
        return;
    }

    for (var i = 0; i < files.length; i++) {
        doc.write('<script type="text/javascript" src="' + files[i] + '"><\/script>');
    }
})();
"""
    }


class WSGICompile(unittest.TestCase):

    def get_app(self, inputs):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"),
            os.path.join(os.path.dirname(__file__)),
            ]
        return webtest.TestApp(
            wsgi.Raw(tree = files.Tree(paths = paths, inputs = inputs)))

    def get_inputApp(self, inputs):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"),
            os.path.join(os.path.dirname(__file__)),
            ]
        return webtest.TestApp(
            wsgi.Input(tree = files.Tree(paths = paths, inputs = inputs)))

    def test_secure_input_app(self):
        app = self.get_inputApp(["test1.js"])
        resp = app.get("/etc/password", expect_errors = True)
        self.assertEqual(resp.status_int, 404)

    def test_compile_raw1(self):
        app = self.get_app(["test1.js"])
        resp = app.get("/")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assertEqual(resp.body, BODIES["test1"])

    def test_compile_raw2(self):
        app = self.get_app(["test1.js"])
        resp = app.get("/?input=test1.js&input=test2.js")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assertEqual(resp.body, BODIES["test2"])

    def test_input1(self):
        app = self.get_inputApp(None)
        resp = app.get("/closure/goog/base.js")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")

    def test_input2(self):
        app = self.get_inputApp(None)
        resp = app.get("/missing.js", expect_errors = True)
        self.assertEqual(resp.status_int, 404)
        self.assertEqual(resp.body, "")

    def get_combined(self, inputs):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"),
            os.path.join(os.path.dirname(__file__)),
            ]
        return webtest.TestApp(wsgi.Combined(
            tree = files.Tree(paths = paths, inputs = inputs)))

    def test_combined1(self):
        app = self.get_combined(["test1.js"])
        resp = app.get("/", expect_errors = True)
        self.assertEqual(resp.status_int, 404)

    def test_combined2(self):
        app = self.get_combined(["test1.js"])
        resp = app.get("/compile")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assertEqual(resp.body, BODIES["test1"])

    def test_combined3(self):
        app = self.get_combined(["test1.js"])
        resp = app.get("/input", expect_errors = True)
        self.assertEqual(resp.status_int, 404)

    def test_combined3(self):
        app = self.get_combined(["test1.js"])
        resp = app.get("/input/closure/goog/base.js")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")

    def get_subApp(self, inputs):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"
                ),
            os.path.join(os.path.dirname(__file__)),
            ]
        urlmap = paste.urlmap.URLMap()
        urlmap["/js"] = wsgi.Combined(
            tree = files.Tree(paths = paths, inputs = inputs))
        return webtest.TestApp(urlmap)

    def test_sub_combined1(self):
        app = self.get_subApp(["test1.js"])
        resp = app.get("/js/compile")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assertEqual(resp.body, BODIES["test1_js"])

    def get_compileApp(self, inputs):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"),
            os.path.join(os.path.dirname(__file__)),
            ]
        app = wsgi.Compile(
            tree = files.Tree(paths = paths, inputs = inputs))
        return webtest.TestApp(app)

    def test_compile1(self):
        app = self.get_compileApp(["test1.js"])
        resp = app.get("/")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assert_(resp.content_length > 0)

    def test_compile2(self):
        # XXX - update test to make sure that we pull in the extra Java Script
        # source specified
        app = self.get_compileApp(["test1.js"])
        resp = app.get("/?input=test1.js&input=test2.js")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        self.assert_(resp.content_length > 0)

    def test_paste_integration1(self):
        paths = [
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"),
            os.path.join(os.path.dirname(__file__)),
            ]
        app = webtest.TestApp(
            wsgi.paste_combined_closure(
                {},
                inputs = "",
                paths = "\n".join(paths),
                )
            )

        resp = app.get("/", expect_errors = True)
        self.assertEqual(resp.status_int, 404)

        resp = app.get("/input/test1.js")
        self.assertEqual(resp.status_int, 200)


class WSGICompile2(unittest.TestCase):
    # We need to fix the closure tree when generating deps.js files as
    # otherwise we will get different output based on the version
    # of the closure library that we are using.

    def setUp(self):
        self.root = tempfile.mkdtemp()

        self.writefile("base.js", "goog.provide('goog');\n")

    def tearDown(self):
        shutil.rmtree(self.root)

    def writefile(self, name, contents):
        filename = os.path.join(self.root, name)
        open(filename, "w").write(contents)
        return filename

    def get_DepsApp(self):
        paths = [
            self.root,
            os.path.join(os.path.dirname(__file__)),
            ]
        return webtest.TestApp(
            wsgi.Deps(tree = files.Tree(paths = paths)))

    def test_deps1(self):
        app = self.get_DepsApp()
        resp = app.get("/")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        expectedBody = open(
            os.path.join(os.path.dirname(__file__), "test_deps1.js")).read()
        self.assertEqual(resp.body, expectedBody)

    def get_combined(self, inputs):
        paths = [
            self.root,
            os.path.join(os.path.dirname(__file__)),
            ]
        return webtest.TestApp(wsgi.Combined(
            tree = files.Tree(paths = paths, inputs = inputs)))

    def test_combined_deps1(self):
        app = self.get_combined(["test1.js"])
        resp = app.get("/deps")
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, "application/javascript")
        expectedBody = open(
            os.path.join(
                os.path.dirname(__file__), "test_deps1.js"
                )
            ).read()
        self.assertEqual(resp.body, expectedBody)


class TestFiles(unittest.TestCase):

    def setUp(self):
        self.root1 = tempfile.mkdtemp()
        self.writefile1("base.js", "goog.provide('goog');\n")

    def tearDown(self):
        shutil.rmtree(self.root1)

    def writefile1(self, name, contents):
        filename = os.path.join(self.root1, name)
        open(filename, "w").write(contents)
        return filename

    def test_tree1(self):
        # If we don't update the tree we don't have access to the path_info
        tree = files.Tree(paths = [self.root1])
        # tree.update()
        self.assertEqual(tree.path_info, None)
        tree.update()
        self.assertNotEqual(tree.path_info, None)

    def test_tree2(self):
        # We need to update the tree
        tree = files.Tree(paths = [self.root1])
        tree.update()
        info = dict(
            [(key, val.GetPath()) for key, val in tree.path_info.items()])
        self.assertEqual(info, {"/base.js": "%s/base.js" % self.root1})

    def test_deps1(self):
        tree = files.Tree(paths = [self.root1])
        self.assertRaises(ValueError, tree.getDeps, "")

    def test_deps2(self):
        # test getDeps with full filename
        filename = self.writefile1("app.js", """goog.provide('app');\n""")
        tree = files.Tree(paths = [self.root1])
        deps = tree.getDeps([filename])
        self.assertEqual(len(deps), 2)
        self.assertEqual(
            [d.GetPath() for d in deps],
            ["%s/base.js" % self.root1, "%s/app.js" % self.root1])

    def test_deps3(self):
        # test getDeps with just the path info
        filename = self.writefile1("app.js", """goog.provide('app');\n""")
        tree = files.Tree(paths = [self.root1])
        deps = tree.getDeps(["app.js"])
        self.assertEqual(len(deps), 2)
        self.assertEqual(
            [d.GetPath() for d in deps],
            ["%s/base.js" % self.root1, "%s/app.js" % self.root1])

    def test_deps4(self):
        # default inputs
        filename = self.writefile1("app.js", """goog.provide('app');\n""")
        tree = files.Tree(paths = [self.root1], inputs = ["app.js"])
        deps = tree.getDeps()
        self.assertEqual(len(deps), 2)
        self.assertEqual(
            [d.GetPath() for d in deps],
            ["%s/base.js" % self.root1, "%s/app.js" % self.root1])

    def test_source1(self):
        filename = self.writefile1("app.js", """goog.provide('app');\n""")
        tree = files.Tree(paths = [self.root1])
        src = tree.getSource("/base.js")
        self.assertEqual(src.GetSource(), "goog.provide('goog');\n")


class ClosureTemplatesTestCase(unittest.TestCase):

    def setUp(self):
        self.root1 = tempfile.mkdtemp()
        self.writefile1("base.js", "goog.provide('goog');\n")
        self.writefile1("soy.js", """goog.provide('soy.StringBuilder');
goog.provide('soy');""")

    def tearDown(self):
        shutil.rmtree(self.root1)

    def writefile1(self, name, contents):
        filename = os.path.join(self.root1, name)
        open(filename, "w").write(contents)
        return filename

    def test_source_scan1(self):
        filename = self.writefile1("test.soy", """{namespace examples.soy}

{template .helloWorld}Hello{/template}""")

        source = files.SoySource(files.Tree(paths = [self.root1]), filename)
        self.assertEqual(source.provides, set(["examples.soy"]))
        self.assertEqual(source.requires, set(["soy", "soy.StringBuilder"]))

    def test_source_scan2(self):
        filename = self.writefile1("test.soy", """{namespace examples.soy2}

{template .helloWorld}Hello{/template}""")

        source = files.SoySource(files.Tree(paths = [self.root1]), filename)
        self.assertEqual(source.provides, set(["examples.soy2"]))
        self.assertEqual(source.requires, set(["soy", "soy.StringBuilder"]))

    def test_source_scan3(self):
        filename = self.writefile1("test.soy", """{namespace examples.soy}
{namespace examples.soy2}

{template .helloWorld}Hello{/template}""")

        class DummyTree(object):
            pass

        source = files.SoySource(DummyTree(), filename)
        self.assertEqual(
            source.provides, set(["examples.soy", "examples.soy2"]))
        self.assertEqual(source.requires, set(["soy", "soy.StringBuilder"]))

    def test_files1(self):
        filename = self.writefile1("test.soy", """{namespace examples.soy}
/** */
{template .helloWorld}
Hello
{/template}
""")

        tree = files.Tree(paths = [self.root1])
        source = tree.getSource("/test.soy")
        self.assertEqual(source.GetSource(), """// This file was automatically generated from test.soy.
// Please don't edit this file by hand.

goog.provide('examples.soy');

goog.require('soy');
goog.require('soy.StringBuilder');


examples.soy.helloWorld = function(opt_data, opt_sb) {
  var output = opt_sb || new soy.StringBuilder();
  output.append('Hello');
  if (!opt_sb) return output.toString();
};
""")


class RecipeTestCase(unittest.TestCase):

    def setUp(self):
        base = tempfile.mkdtemp("buildoutSetUp")
        self.base = os.path.realpath(base)

        self.cwd = os.getcwd()

        # XXX - when we run the recipe in the test buildout upgrades itself and
        # installs a new version of setuptools and zc.buildout but never
        # reuses the ones installed by the buildout to generate the test
        # scripts.
        self.builddir = os.path.join(self.base, "_TEST_")
        os.mkdir(self.builddir)
        self.media = os.path.join(self.builddir, "media")
        os.mkdir(self.media)

        # Nothing so far
        compiled = os.listdir(self.media)
        self.assertEqual(len(compiled), 0)

        deggs = os.path.join(self.builddir, "develop-eggs")
        os.mkdir(deggs)

        zc.buildout.testing.install_develop("setuptools", deggs)
        zc.buildout.testing.install_develop("zc.buildout", deggs)
        zc.buildout.testing.install_develop("pwt.closure", deggs)
        zc.buildout.testing.install_develop("Paste", deggs)
        zc.buildout.testing.install_develop("WebOb", deggs)

    def tearDown(self):
        shutil.rmtree(self.base)
        os.chdir(self.cwd)

    def test_compiled_recipe(self):
        closure_path = os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"
                )
            )

        open(os.path.join(self.builddir, "buildout.cfg"), "w").write("""
[buildout]
parts = compiled.js

[compiled.js]
recipe = pwt.closure:compile
paths =
    %(closure)s
    %(pwt.closure)s

inputs = %(inputs)s

outputdir = %(media)s
""" % {"closure": closure_path,
       "pwt.closure": os.path.dirname(__file__),
       "inputs": os.path.join(os.path.dirname(__file__), "test1.js"),
       "media": self.media,
       })

        os.chdir(self.builddir)
        config = [
            ("buildout", "log-level", "WARNING"),
            ("buildout", "offline", "true"),
            ("buildout", "newest", "false"),
            ]
        zc.buildout.buildout.Buildout(
            "buildout.cfg", config, user_defaults = False,
            ).bootstrap([])

        buildout = os.path.join(self.builddir, "bin", "buildout")
        os.system(buildout)

        compiled = os.listdir(self.media)
        self.assertEqual(len(compiled), 1)
        self.assert_(compiled[0].endswith(".js"))

    def test_deps_recipe(self):
        closure_path = os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                "..", "..", "..", "checkouts", "closure"
                )
            )

        open(os.path.join(self.builddir, "buildout.cfg"), "w").write("""
[buildout]
parts = deps.js

[deps.js]
recipe = pwt.closure:deps
paths =
    %(closure)s
    %(pwt.closure)s

deps_js = %(media)s/deps.js
""" % {"closure": closure_path,
       "pwt.closure": os.path.dirname(__file__),
       "inputs": os.path.join(os.path.dirname(__file__), "test1.js"),
       "media": self.media,
       })

        os.chdir(self.builddir)
        config = [
            ("buildout", "log-level", "WARNING"),
            ("buildout", "offline", "true"),
            ("buildout", "newest", "false"),
            ]
        zc.buildout.buildout.Buildout(
            "buildout.cfg", config, user_defaults = False,
            ).bootstrap([])

        buildout = os.path.join(self.builddir, "bin", "buildout")
        os.system(buildout)

        deps = os.listdir(self.media)
        self.assertEqual(len(deps), 1)
        self.assertEqual(deps[0], "deps.js")



import hashlib
import os.path

import files

def getTree(options):
    config = options.copy()
    config["inputs"] = options.get("inputs", "").split()
    config["paths"] = options["paths"].split()
    compiler_jar = options.get("compiler_jar", None)
    if compiler_jar:
        config["compiler_jar"] = compiler_jar
    config["compiler_flags"] = options.get("compiler_flags", "").split()
    # need to configure Jinja2 environment if appropriate
    try:
        config["jinja2.environment"] = files.parse_environment(options)
    except (KeyError, ValueError), err:
        pass

    return files.Tree(**config)


class CompileRecipe(object):

    def __init__(self, buildout, name, options):
        self.buildout = buildout
        self.name = name
        self.options = options

        tree = getTree(options)

        self.compiled_code = tree.getCompiledSource()

        md5name = hashlib.md5()
        md5name.update(self.compiled_code)
        self.options["filename"] = self.filename = md5name.hexdigest() + ".js"

    def install(self):
        filename = os.path.join(self.options["outputdir"], self.filename)
        open(filename, "w").write(self.compiled_code)
        return (filename,)

    def update(self):
        return self.install()


class DepsRecipe(object):

    def __init__(self, buildout, name, options):
        self.buildout = buildout
        self.name = name
        self.options = options

        tree = getTree(options)

        self.compiled_code = files.MakeDepsFile(tree)

    def install(self):
        open(self.options["deps_js"], "w").write(self.compiled_code)
        return (self.options["deps_js"],)

    def update(self):
        return self.install()



import hashlib
import optparse
import sys

import files

def main(args = None, output = None):
    if output is None:
        output = sys.stdout

    parser = optparse.OptionParser()

    parser.add_option(
        "--outputdir", dest = "outputdir", default = None,
        metavar = "OUTPUTDIR")

    parser.add_option(
        "--input", dest = "inputs", action = "append", default = [],
        metavar = "INPUTS",
        )

    parser.add_option(
        "--root", dest = "paths", action = "append", default = [],
        metavar = "ROOTS",
        )

    options, inputs = parser.parse_args(args)

    # Directory to save the comiled Java Script to
    if not options.outputdir:
        raise ValueError("Missing outputdir")

    # We can list our inputs on the command line without the --input option
    options.inputs.extend(inputs)

    # need to configure Jinja2 environment if appropriate
    try:
        options["jinja2.environment"] = files.parse_environment(options)
    except (KeyError, ValueError), err:
        pass

    tree = files.Tree(**options)
    compiled_code = tree.getCompiledSource(options.inputs)

    md5 = haslib.md5()
    md5.update(compiled_code)

    open(
        os.path.join(options.outputdir, md5.hexdigest() + ".js"),
        "w").\
        write(compiled_code)


if __name__ == "__main__":
    main(sys.argv[1:])



"""
pwt.closure is a tool to compile and manage JavaScript dependencies of your
web application.
"""
from setuptools import find_packages, setup

setup(
    name = "pwt.closure",
    version = "0.2",

    author = "Michael Kerrin",
    author_email = "michael.kerrin@gmail.com",
    license = "BSD",
    description = __doc__,
    long_description = open("README.txt").read(),
    url = "https://github.com/mkerrin/pwt.closure",

    packages = find_packages("src"),
    package_dir = {"": "src"},
    namespace_packages = ["pwt"],

    install_requires = [
        "setuptools",
        ],

    extras_require = {
        "test": [
            "WebTest",
            "Paste",
            "zc.buildout",
            ],
        # You must install this extra requirement so that I don't have to
        # worry about installing development versions of WebOb and Paste
        # during the installation of recipes in buildout.
        "web": [
            "WebOb",
            "Paste",
            ],
        },

    entry_points = {
        "paste.app_factory": [
            "main = pwt.closure.wsgi:paste_combined_closure",
            ],
        "console_scripts": [
            "closurebuilder = pwt.closure.cli:main",
            ],
        "zc.buildout": [
            "compile = pwt.closure.recipe:CompileRecipe",
            "deps = pwt.closure.recipe:DepsRecipe",
            ],
        },

    include_package_data = True,
    zip_safe = False,
    )
